package cn.ecpark.service.usergw.config;

import java.io.InputStream;
import java.net.URI;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import org.bouncycastle.cert.crmf.RegTokenControl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.gateway.config.GatewayProperties;
import org.springframework.cloud.gateway.filter.FilterDefinition;
import org.springframework.cloud.gateway.filter.NettyWriteResponseFilter;
import org.springframework.cloud.gateway.handler.predicate.PredicateDefinition;
import org.springframework.cloud.gateway.route.RouteDefinition;
import org.springframework.cloud.gateway.route.RouteDefinitionLocator;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.context.annotation.Bean;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Component;
import org.springframework.web.util.UriBuilder;
import org.yaml.snakeyaml.Yaml;

import lombok.extern.slf4j.Slf4j;
import reactor.core.publisher.Flux;

/**
 * ConfigGateway
 */
@Component
@Slf4j
public class ConfigGateway implements RouteDefinitionLocator {

  List<MediaType> mediaTypes = new ArrayList<>();

  @Autowired
  private ApplicationContext applicationContext;

  @Override
  public Flux<RouteDefinition> getRouteDefinitions() {
    
    NettyWriteResponseFilter nwrf = applicationContext.getBean(NettyWriteResponseFilter.class);
    
    Object inputStream = null;
    String[] gatewayConfigPathList = { "gateway.yml", "gateway.yaml", "Gateway.yml", "Gateway.yaml" };
    for (String gatewayConfigPath : gatewayConfigPathList) {
      inputStream = this.getClass().getClassLoader().getResourceAsStream(gatewayConfigPath);
      if(inputStream != null) {
        break;
      }
    }
    
    if (inputStream != null) {
      Map<String, Object> configYaml = new Yaml().load((InputStream) inputStream);
      if (configYaml != null) {
        Map<String, Object> configDefault = (Map<String, Object>) configYaml.get("default");
        Map<String, Object> configDubbo = (Map<String, Object>) configYaml.get("dubbo");
        return Flux.fromIterable(ConfigProperties.getConfigDefault(configDefault));
      }
    }
    return null;
    // return Flux.fromIterable(it);
  }
}