/*
 * Copyright 2013-2019 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package cn.ecpark.service.usergw.config;

import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import javax.validation.Valid;
import javax.validation.constraints.NotNull;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.cloud.gateway.filter.FilterDefinition;
import org.springframework.cloud.gateway.handler.predicate.PredicateDefinition;
import org.springframework.cloud.gateway.route.RouteDefinition;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.PropertySource;
import org.springframework.http.MediaType;
import org.springframework.validation.annotation.Validated;

import lombok.extern.slf4j.Slf4j;

/**
 * ConfigProperties
 */
@Validated
@Slf4j
public class ConfigProperties {
    
    public static List<RouteDefinition> getConfigDefault(Map<String, Object> configDefault) {
        if (configDefault != null) {
            List<RouteDefinition> RDList = new ArrayList<>();

            List<FilterDefinition> filters = new ArrayList<>();
            List<PredicateDefinition> predicates = new ArrayList<>();

            // default-filters: 下的相关设置
            Object unknownDefaultFilters = configDefault.get("default-filters");
            if (unknownDefaultFilters != null) {
                List<String> defaultFiltersYaml = (ArrayList<String>) unknownDefaultFilters;
                for (String filterString : defaultFiltersYaml) {
                    filters.add(new FilterDefinition(filterString));
                }
            }

            Object unknownRoutes = configDefault.get("routes");
            if (unknownRoutes != null) {
                List<LinkedHashMap> routes = (ArrayList<LinkedHashMap>) unknownRoutes;
                for (LinkedHashMap iter : routes) {
                    RouteDefinition rd = new RouteDefinition();
                    // 设置id
                    Object id = iter.get("id");
                    if (id != null) {
                        rd.setId((String) id);
                    }

                    // 设置uri
                    Object uri = iter.get("uri");
                    if (uri != null) {
                        rd.setUri(URI.create((String) uri));
                    }

                    // 设置uri
                    Object order = iter.get("order");
                    if (order != null) {
                        rd.setOrder((int) order);
                    }

                    // predicates: 下的相关属性
                    List<String> unknownPredicatesYaml = (ArrayList<String>) iter.get("predicates");
                    if (unknownPredicatesYaml != null) {
                        List<String> predicatesYaml = (ArrayList<String>) unknownPredicatesYaml;
                        for (String predicateString : predicatesYaml) {
                            predicates.add(new PredicateDefinition(predicateString));
                        }
                    }

                    // filters: 下的相关属性
                    List<String> unknownFiltersYaml = (ArrayList<String>) iter.get("filters");
                    if (unknownFiltersYaml != null) {
                        List<String> filtersYaml = (ArrayList<String>) unknownFiltersYaml;
                        for (String filterString : filtersYaml) {
                            filters.add(new FilterDefinition(filterString));
                        }
                    }

                    rd.setPredicates(predicates);
                    rd.setFilters(filters);

                    RDList.add(rd);
                }

                return RDList;
            }
        }

        return null;
    }

}
